#include <iostream>

#include <exception>
#include <ctime>
#include <string>
#include <cstring>
#include <fstream>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>

#include "Eapi.h"
#include "Eapi_Seco.h"

#define NO_OP           0
#define EAPI_LIBRARY    1

#define PARAMETER_ERROR         0

#define GET_SPEC_VERSION		1
#define GET_BOARD_PLATFORM_REV	2
#define GET_BOARD_LIB_VERSION   3
#define GET_HWMON_CPU_TEMP      4
#define GET_HWMON_CHIPSET_TEMP  5
#define GET_HWMON_FAN_CPU       6
#define GET_HWMON_FAN_SYSTEM    7
#define GET_HWMON_VOLTAGE_VCORE 8
#define GET_HWMON_VOLTAGE_2V5   9
#define GET_HWMON_VOLTAGE_3V3   10
#define GET_HWMON_VOLTAGE_VBAT  11
#define GET_HWMON_VOLTAGE_5V    12
#define GET_HWMON_VOLTAGE_5VSB  13
#define GET_HWMON_VOLTAGE_12V   14
#define GET_BOARD_BOOT_COUNTER  23
// -----------------------------------
#define GET_BOARD_MANUFACTURER  15
#define GET_BOARD_NAME			16
#define GET_BOARD_REVISION		17
#define GET_BOARD_SERIAL		18
#define GET_BOARD_BIOS_REVISION 20
#define GET_BOARD_HW_REVISION   21
#define GET_BOARD_PLATFORM_TYPE 22
// -----------------------------------
#define GET_LCD_BACKLIGHT		30
#define SET_LCD_BACKLIGHT		31
#define SET_LCD_BCKL_ENA        32
// -----------------------------------
#define NVS_SIZE                40
#define NVS_READ                41
#define NVS_WRITE               42
// -----------------------------------
#define WDOG_CAP				50
#define WDOG_TRIGGER			51
#define WDOG_START              52
#define WDOG_STOP               53
#define WDOG_START_CUSTOM       54
// -----------------------------------
#define I2C_CAP					60
#define I2C_WRITE_READ_RAW      61
#define I2C_READ                62
#define I2C_WRITE               63
#define I2C_PROBE				64
// -----------------------------------
#define GPIO_GET_DIRECTION_CAP	70
#define GPIO_GET_DIRECTION		71
#define GPIO_SET_DIRECTION		72
#define GPIO_GET_LEVEL			73
#define GPIO_SET_LEVEL			74
#define GPIO_GET_SERIAL_CFG     75
#define GPIO_SET_SERIAL_CFG     76
// -----------------------------------
#define FAN_ENABLE              80
#define FAN_DISABLE             81
#define FAN_GET_CYCLE           82
#define FAN_SET_CYCLE           83
#define FAN_GET_DUTY            84
#define FAN_SET_DUTY            85
// -----------------------------------
#define SPI_CAP					90
#define SPI_WRITE_READ_RAW      91
#define SPI_READ                92
#define SPI_WRITE               93
// -----------------------------------
#define SMBUS_READ_BYTE         100
#define SMBUS_WRITE_BYTE        101
#define SMBUS_READ_WORD         102
#define SMBUS_WRITE_WORD        103
// -----------------------------------
#define GET_RESET_CAUSE         104
#define CLEAR_RESET_CAUSE       105
// -----------------------------------
#define HELP				    200
#define TEST_01				    201
#define TEST_02				    202
#define TEST_03				    203
#define TEST_04				    204
// -----------------------------------

using namespace std;

char cOption;
char cOption_2;

void ShowStatus(EApiStatus_t Ret)
{
	switch(Ret)
	{
		case EAPI_STATUS_NOT_INITIALIZED:
			 cout << "EAPI_STATUS_NOT_INITIALIZED" << endl;
			 break;
		case EAPI_STATUS_INITIALIZED:
			cout << "EAPI_STATUS_INITIALIZED" << endl;
			break;
		case EAPI_STATUS_ALLOC_ERROR:
			cout << "EAPI_STATUS_ALLOC_ERROR" << endl;
			break;
		case EAPI_STATUS_DRIVER_TIMEOUT:
			cout << "EAPI_STATUS_DRIVER_TIMEOUT" << endl;
			break;
		case EAPI_STATUS_INVALID_PARAMETER:
			cout << "EAPI_STATUS_INVALID_PARAMETER" << endl;
			break;
		case EAPI_STATUS_INVALID_BLOCK_ALIGNMENT:
			cout << "EAPI_STATUS_INVALID_BLOCK_ALIGNMENT" << endl;
			break;
		case EAPI_STATUS_INVALID_BLOCK_LENGTH:
			cout << "EAPI_STATUS_INVALID_BLOCK_LENGTH" << endl;
            break;
		case EAPI_STATUS_INVALID_DIRECTION:
			cout << "EAPI_STATUS_INVALID_DIRECTION" << endl;
			break;
		case EAPI_STATUS_INVALID_BITMASK:
			 cout << "EAPI_STATUS_INVALID_BITMASK" << endl;
			break;
		case EAPI_STATUS_RUNNING:
            cout << "EAPI_STATUS_RUNNING" << endl;
			break;
		case EAPI_STATUS_UNSUPPORTED:
			cout << "EAPI_STATUS_UNSUPPORTED" << endl;
			break;
		case EAPI_STATUS_NOT_FOUND:
			cout << "EAPI_STATUS_NOT_FOUND" << endl;
			break;
		case EAPI_STATUS_TIMEOUT:
			cout << "EAPI_STATUS_TIMEOUT" << endl;
			break;
		case EAPI_STATUS_BUSY_COLLISION:
			cout << "EAPI_STATUS_BUSY_COLLISION" << endl;
			break;
		case EAPI_STATUS_READ_ERROR:
			cout << "EAPI_STATUS_READ_ERROR" << endl;
			break;
		case EAPI_STATUS_WRITE_ERROR:
			cout << "EAPI_STATUS_WRITE_ERROR" << endl;
			break;
		case EAPI_STATUS_MORE_DATA:
			cout << "EAPI_STATUS_MORE_DATA" << endl;
			break;
		case EAPI_STATUS_ERROR:
			cout << "EAPI_STATUS_ERROR" << endl;
			break;
		case EAPI_STATUS_SUCCESS:
			break;
		default:
			cout << "EAPI_STATUS_ERROR Generic" << endl;
			break;
	}
}

int elev_pot(int valore, int esp)
{
    int iRet = 1;

    for(int i = 0; i < esp; i++)
    {
        iRet *= valore;
    }

    return iRet;
}

int parser(int argc, char* argv[], unsigned char* command, uint32_t* param, unsigned char* num_param)
{
    if(argc > 1)
    {
            if(argv[1][0] != '/' && argv[1][0] != '-')
            {
                *command = PARAMETER_ERROR;
                return 0;
            }

            cOption     = (char)tolower(argv[1][1]);
            cOption_2   = (char)tolower(argv[1][2]);

            switch(cOption)
            {
                case 'l':
                switch(cOption_2)
                {
                    case's': *command = GET_SPEC_VERSION;		break;
                    case'p': *command = GET_BOARD_PLATFORM_REV; break;
                    case'l': *command = GET_BOARD_LIB_VERSION;	break;
                    case'c': *command = GET_HWMON_CPU_TEMP;		break;
                    case'y': *command = GET_HWMON_CHIPSET_TEMP;	break;
                    case'f': *command = GET_HWMON_FAN_CPU;		break;
                    case'a': *command = GET_HWMON_FAN_SYSTEM;	break;
                    case'o': *command = GET_HWMON_VOLTAGE_VCORE;break;
                    case'2': *command = GET_HWMON_VOLTAGE_2V5;	break;
                    case'3': *command = GET_HWMON_VOLTAGE_3V3;	break;
                    case'w': *command = GET_HWMON_VOLTAGE_VBAT;	break;
                    case'5': *command = GET_HWMON_VOLTAGE_5V;	break;
                    case'v': *command = GET_HWMON_VOLTAGE_5VSB;	break;
                    case'1': *command = GET_HWMON_VOLTAGE_12V;	break;
                    case'u': *command = GET_BOARD_BOOT_COUNTER;	break;
					// --------------------------------------------------
                    case'm': *command = GET_BOARD_MANUFACTURER; break;
                    case'n': *command = GET_BOARD_NAME;			break;
                    case'r': *command = GET_BOARD_REVISION;		break;
                    case'e': *command = GET_BOARD_SERIAL;		break;
                    case'b': *command = GET_BOARD_BIOS_REVISION;break;
                    case'h': *command = GET_BOARD_HW_REVISION;	break;
                    case't': *command = GET_BOARD_PLATFORM_TYPE;break;
                    default: *command = PARAMETER_ERROR;		break;
			    }
                break;

                case 'v':
                switch(cOption_2)
                {
                    case'g': *command = GET_LCD_BACKLIGHT; break;
                    case's': *command = SET_LCD_BACKLIGHT; break;
                    case'b': *command = SET_LCD_BCKL_ENA;  break;
                    default: *command = PARAMETER_ERROR;   break;
                }
                break;

                case 's':
                switch(cOption_2)
                {
                    case's': *command = NVS_SIZE;		 break;
                    case'r': *command = NVS_READ;		 break;
                    case'w': *command = NVS_WRITE;		 break;
                    default: *command = PARAMETER_ERROR; break;
                }
                break;

                case 'w':
                switch(cOption_2)
                {
                    case'c': *command = WDOG_CAP;		  break;
                    case't': *command = WDOG_TRIGGER;	  break;
                    case's': *command = WDOG_START;		  break;
                    case'd': *command = WDOG_STOP;		  break;
                    case'r': *command = WDOG_START_CUSTOM;break;
                    default: *command = PARAMETER_ERROR;  break;
                }
                break;

                case 'i':
                switch(cOption_2)
                {
                    case'c': *command = I2C_CAP;			break;
                    case'r': *command = I2C_READ;			break;
                    case'w': *command = I2C_WRITE;			break;
                    case'e': *command = I2C_WRITE_READ_RAW; break;
                    case'p': *command = I2C_PROBE;			break;
                    default: *command = PARAMETER_ERROR;	break;
                }
                break;

                case 'g':
                switch(cOption_2)
                {
                    case'p': *command = GPIO_GET_DIRECTION_CAP; break;
                    case'g': *command = GPIO_GET_DIRECTION;		break;
                    case's': *command = GPIO_SET_DIRECTION;		break;
                    case'c': *command = GPIO_GET_LEVEL;			break;
                    case'f': *command = GPIO_SET_LEVEL;			break;
                    case'r': *command = GPIO_GET_SERIAL_CFG;    break;
                    case'q': *command = GPIO_SET_SERIAL_CFG;    break;
					default: *command = PARAMETER_ERROR;		break;
                }
                break;

                case 'f':
                switch(cOption_2)
                {
                    case'e': *command = FAN_ENABLE;		break;
                    case'd': *command = FAN_DISABLE;	break;
                    case'g': *command = FAN_GET_CYCLE;  break;
                    case's': *command = FAN_SET_CYCLE;  break;
                    case'u': *command = FAN_GET_DUTY;	break;
                    case'c': *command = FAN_SET_DUTY;	break;
                    default: *command = PARAMETER_ERROR;break;
                }
                break;

                case 'p':
                switch(cOption_2)
                {
                    case'c': *command = SPI_CAP;			break;
                    case'r': *command = SPI_READ;			break;
                    case'w': *command = SPI_WRITE;			break;
                    case'e': *command = SPI_WRITE_READ_RAW; break;
                    default: *command = PARAMETER_ERROR;	break;
                }
                break;

                case 'm':
                switch(cOption_2)
                {
                    case'r': *command = SMBUS_READ_BYTE;	break;
                    case'w': *command = SMBUS_WRITE_BYTE;	break;
                    case'e': *command = SMBUS_READ_WORD;	break;
                    case'o': *command = SMBUS_WRITE_WORD;	break;
                    default: *command = PARAMETER_ERROR;	break;
                }
                break;

                case 'r':
                switch(cOption_2)
                {
                    case'g': *command = GET_RESET_CAUSE;	break;
                    case'c': *command = CLEAR_RESET_CAUSE;	break;
                    default: *command = PARAMETER_ERROR;	break;
                }
                break;

                case 't':
                switch(cOption_2)
                {
                    case'1': *command = TEST_01;	break;
                    case'2': *command = TEST_02;	break;
                    case'3': *command = TEST_03;	break;
                    case'4': *command = TEST_04;	break;
                    default: *command = PARAMETER_ERROR;	break;
                }
                break;

                case 'h':
                case '?':
                *command = HELP;
                printf("\nEAPI_Test Manual:\n");
                printf("EAPI_Test -<Option_1><Option_2> {<param_1> ... <param_n>}\n\n");
                printf("           -lm -> Call EApiBoardGetStringA to get EAPI_ID_BOARD_MANUFACTURER_STR\n");
                printf("                  No parameter reqired\n\n");
                printf("           -ln -> Call EApiBoardGetStringA to get EAPI_ID_BOARD_NAME_STR\n");
                printf("                  No parameter reqired\n\n");
                printf("           -le -> Call EApiBoardGetStringA to get EAPI_ID_BOARD_SERIAL_STR\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lb -> Call EApiBoardGetStringA to get EAPI_ID_BOARD_BIOS_REVISION_STR\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lt -> Call EApiBoardGetStringA to get EAPI_ID_BOARD_PLATFORM_TYPE_STR\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lr -> Call EApiBoardGetStringA to get EAPI_ID_BOARD_REVISION_STR\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lh -> Call EApiBoardGetStringA to get EAPI_ID_BOARD_HW_REVISION_STR\n");
                printf("                  No parameter reqired\n\n");
                printf("           -ls -> Call EApiBoardGetValue to get EAPI_ID_GET_EAPI_SPEC_VERSION\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lp -> Call EApiBoardGetValue to get EAPI_ID_BOARD_PLATFORM_REV_VAL\n");
                printf("                  No parameter reqired\n\n");
                printf("           -ll -> Call EApiBoardGetValue to get EAPI_ID_BOARD_LIB_VERSION_VAL\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lc -> Call EApiBoardGetValue to get EAPI_ID_HWMON_CPU_TEMP\n");
                printf("                  No parameter reqired\n\n");
                printf("           -ly -> Call EApiBoardGetValue to get EAPI_ID_HWMON_CHIPSET_TEMP\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lo -> Call EApiBoardGetValue to get EAPI_ID_HWMON_VOLTAGE_VCORE\n");
                printf("                  No parameter reqired\n\n");
                printf("           -l2 -> Call EApiBoardGetValue to get EAPI_ID_HWMON_VOLTAGE_2V5\n");
                printf("                  No parameter reqired\n\n");
                printf("           -l3 -> Call EApiBoardGetValue to get EAPI_ID_HWMON_VOLTAGE_3V3\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lw -> Call EApiBoardGetValue to get EAPI_ID_HWMON_VOLTAGE_VBAT\n");
                printf("                  No parameter reqired\n\n");
                printf("           -l5 -> Call EApiBoardGetValue to get EAPI_ID_HWMON_VOLTAGE_5V\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lv -> Call EApiBoardGetValue to get EAPI_ID_HWMON_VOLTAGE_5VSB\n");
                printf("                  No parameter reqired\n\n");
                printf("           -l1 -> Call EApiBoardGetValue to get EAPI_ID_HWMON_VOLTAGE_12V\n");
                printf("                  No parameter reqired\n\n");
                printf("           -lf -> Call EApiBoardGetValue to get EAPI_ID_HWMON_FAN_CPU\n");
                printf("                  No parameter reqired\n\n");
                printf("           -la -> Call EApiBoardGetValue to get EAPI_ID_HWMON_FAN_SYSTEM\n");
                printf("                  No parameter reqired\n");
                printf("           -lu -> Call EApiBoardGetValue to get EAPI_ID_BOARD_BOOT_COUNTER_VAL\n");
                printf("                  No parameter reqired\n");
                printf("\n           -------------------------------------------------------------------------\n\n");
                printf("           -vg -> Call EApiVgaGetBacklightBrightness to get backlight brightness\n");
                printf("                  No parameter reqired\n\n");
                printf("           -vs -> Call EApiVgaSetBacklightBrightness to set backlight brightness\n");
                printf("                  <param_1> -> Backlight brightness percentage value (between 0 and EAPI_BACKLIGHT_SET_BRIGHTEST)\n\n");
                printf("           -vb -> Call EApiVgaSetBacklightEnable to enable or disable backlight\n");
                printf("                  <param_1> -> 0 to disable and 1 to enable\n");
                printf("\n           -------------------------------------------------------------------------\n\n");
                printf("           -ss -> Call EApiStorageCap to get NVS capabilities\n");
                printf("                  No parameter reqired\n\n");
                printf("           -sr -> Call EApiStorageAreaRead to read data from the selected user data area\n");
                printf("                  <param_1> -> Number of bytes to read\n");
                printf("                  <param_2> -> Storage area start address offset in bytes\n\n");
                printf("           -sw -> Call EApiStorageAreaWrite to write data to the selected user data area.\n");
                printf("                  <param_1> -> Number of bytes to write\n");
                printf("                  <param_2> -> Storage area start address offset in bytes\n");
                printf("                  <param_3> ... <param_n> -> Bytes to write\n");
                printf("\n           -------------------------------------------------------------------------\n\n");
                printf("           -wc -> Call EApiWDogGetCap to get the capabilities of the watchdog timer\n");
                printf("                  No parameter reqired\n\n");
                printf("           -wt -> Call EApiWDogTrigger to trigger the watchdog timer\n");
                printf("                  No parameter reqired\n\n");
                printf("           -ws -> Call EApiWDogStart to start the watchdog timer and set the parameters\n");
                printf("                  <param_1> -> Delay Timeout in msec\n");
                printf("                  <param_2> -> Event Timeout in msec\n");
                printf("                  <param_3> -> Reset Timeout in msec\n\n");
                printf("           -wd -> Call EApiWDogStop to get xxx\n");
                printf("                  No parameter reqired\n\n");
                printf("           -wr -> Call EApiWDog_SetConfigAndStart to start the watchdog timer and set the extended parameters\n");
                printf("                  <param_1> -> Delay Timeout in msec\n");
                printf("                  <param_2> -> Event Timeout in msec\n");
                printf("                  <param_3> -> Reset Timeout in msec\n");
                printf("                  <param_4> -> Event after reset timeout\n");
                printf("                               0 for signal\n");
                printf("                               1 for system reset\n");
                printf("                               2 for power button 1 sec\n");
                printf("                               3 for power button 4 sec\n");
                printf("                  <param_5> -> Trigger mode\n");
                printf("                               0 for software trigger\n");
                printf("                               1 for hardware trigger\n");
                printf("\n           -------------------------------------------------------------------------\n\n");
                printf("           -ic -> Call EApiI2CGetBusCap for I2C bus to get the capabilities\n");
                printf("                  <param_1> -> I2C bus:\n");
                printf("                               0 for EAPI_ID_I2C_EXTERNAL\n");
                printf("                               1 for EAPI_ID_I2C_LVDS_1\n");
                printf("                               2 for EAPI_ID_I2C_LVDS_2\n");
                printf("                               3 for EAPI_ID_I2C_SPI\n");
                printf("                               4 for EAPI_ID_I2C_INTERNAL\n");
                printf("           -ir -> Call EApiI2CReadTransfer for I2C bus to read byte from register\n");
                printf("                  <param_1> -> I2C bus:\n");
                printf("                               0 for EAPI_ID_I2C_EXTERNAL\n");
                printf("                               1 for EAPI_ID_I2C_LVDS_1\n");
                printf("                               2 for EAPI_ID_I2C_LVDS_2\n");
                printf("                               3 for EAPI_ID_I2C_SPI\n");
                printf("                               4 for EAPI_ID_I2C_INTERNAL\n");
                printf("                  <param_2> -> Device Address\n");
                printf("                  <param_3> -> Register\n\n");
                printf("           -iw -> Call EApiI2CWriteTransfer for I2C bus to write byte to register xxx\n");
                printf("                  <param_1> -> I2C bus:\n");
                printf("                               0 for EAPI_ID_I2C_EXTERNAL\n");
                printf("                               1 for EAPI_ID_I2C_LVDS_1\n");
                printf("                               2 for EAPI_ID_I2C_LVDS_2\n");
                printf("                               3 for EAPI_ID_I2C_SPI\n");
                printf("                               4 for EAPI_ID_I2C_INTERNAL\n");
                printf("                  <param_2> -> Device Address\n");
                printf("                  <param_3> -> Register\n");
                printf("                  <param_4> -> Register value to write\n\n");
                printf("           -ie -> Call EApiI2CWriteReadRaw for I2C bus for read and write operations\n");
                printf("                  <param_1> -> I2C bus:\n");
                printf("                               0 for EAPI_ID_I2C_EXTERNAL\n");
                printf("                               1 for EAPI_ID_I2C_LVDS_1\n");
                printf("                               2 for EAPI_ID_I2C_LVDS_2\n");
                printf("                               3 for EAPI_ID_I2C_SPI\n");
                printf("                               4 for EAPI_ID_I2C_INTERNAL\n");
                printf("                  <param_2> -> Device Address\n");
                printf("                  <param_3> -> Number of bytes to write\n");
                printf("                  <param_4> -> Number of bytes to read\n");
                printf("                  <param_5> ... <param_n> -> Byte to write\n\n");
                printf("           -ip -> Call EApiI2CProbeDevice for I2C bus to test device present\n");
                printf("                  <param_1> -> I2C bus:\n");
                printf("                               0 for EAPI_ID_I2C_EXTERNAL\n");
                printf("                               1 for EAPI_ID_I2C_LVDS_1\n");
                printf("                               2 for EAPI_ID_I2C_LVDS_2\n");
                printf("                               3 for EAPI_ID_I2C_SPI\n");
                printf("                               4 for EAPI_ID_I2C_INTERNAL\n");
                printf("                  <param_2> -> Device Address\n");
                printf("\n           -------------------------------------------------------------------------\n\n");
                printf("           -gp -> Call EApiGPIOGetDirectionCaps to get the capabilities of the current GPIO implementation from the selected GPIO interface\n");
                printf("                  <param_1> -> Carrier boards where CPU module is connected\n");
                printf("                               0 for without carrier boars\n");
                printf("                               1 for M799\n");
                printf("                               2 for M901\n");
                printf("                               3 for 0A19\n");
                printf("                               4 for 0A42\n");
                printf("                               5 for 0857\n\n");
                printf("                  <param_2> -> ID (see 9.1.1 in COM_EAPI_R1_0.pdf\n");
                printf("           -gg -> Call EApiGPIOGetDirection to get the current configuration of the selected GPIO ports\n");
                printf("                  <param_1> -> Carrier boards where CPU module is connected\n");
                printf("                               0 for without carrier boars\n");
                printf("                               1 for M799\n");
                printf("                               2 for M901\n");
                printf("                               3 for 0A19\n");
                printf("                               4 for 0A42\n");
                printf("                               5 for 0857\n\n");
                printf("                  <param_2> -> ID (see 9.1.1 in COM_EAPI_R1_0.pdf\n");
                printf("           -gs -> Call EApiGPIOSetDirection to set the configuration for the selected GPIO ports\n");
                printf("                  <param_1> -> Carrier boards where CPU module is connected\n");
                printf("                               0 for without carrier boars\n");
                printf("                               1 for M799\n");
                printf("                               2 for M901\n");
                printf("                               3 for 0A19\n");
                printf("                               4 for 0A42\n");
                printf("                               5 for 0857\n");
                printf("                  <param_2> -> ID (see 9.1.1 in COM_EAPI_R1_0.pdf\n");
                printf("                  <param_3> -> Level value mask to set\n\n");
                printf("           -gc -> Call EApiGPIOGetLevel to get level for the selected GPIO ports\n");
                printf("                  <param_1> -> Carrier boards where CPU module is connected\n");
                printf("                               0 for without carrier boars\n");
                printf("                               1 for M799\n");
                printf("                               2 for M901\n");
                printf("                               3 for 0A19\n");
                printf("                               4 for 0A42\n");
                printf("                               5 for 0857\n");
                printf("                  <param_2> -> ID (see 9.1.1 in COM_EAPI_R1_0.pdf\n");
                printf("           -gf -> Call EApiGPIOSetLevel to set level for the selected GPIO ports\n");
                printf("                  <param_1> -> Carrier boards where CPU module is connected\n");
                printf("                               0 for without carrier boars\n");
                printf("                               1 for M799\n");
                printf("                               2 for M901\n");
                printf("                               3 for 0A19\n");
                printf("                               4 for 0A42\n");
                printf("                               5 for 0857\n");
                printf("                  <param_2> -> ID (see 9.1.1 in COM_EAPI_R1_0.pdf\n");
                printf("                  <param_3> -> Level value mask to set\n\n");
                printf("           -gr -> Call EApiGPIOGetSerialCfg to get serial port configuration\n");
                printf("                  <param_1> -> Carrier boards where CPU module is connected\n");
                printf("                               0 for without carrier boars\n");
                printf("                               1 for M799\n");
                printf("                               2 for M901\n");
                printf("                               3 for 0A19\n");
                printf("                               4 for 0A42\n");
                printf("                               5 for 0857\n\n");
                printf("           -gq -> Call EApiGPIOSetSerialCfg to set serial port configuration\n");
                printf("                  <param_1> -> Carrier boards where CPU module is connected\n");
                printf("                               0 for without carrier boars\n");
                printf("                               1 for M799\n");
                printf("                               2 for M901\n");
                printf("                               3 for 0A19\n");
                printf("                               4 for 0A42\n");
                printf("                               5 for 0857\n");
                printf("                  <param_2> -> Carrier boards where CPU module is connected\n");
                printf("                               161 for RS232\n");
                printf("                               162 for RS422\n");
                printf("                               163 for RS485TX\n");
                printf("                               164 for RS485RX\n");
                printf("\n           -------------------------------------------------------------------------\n\n");
                printf("           -fe -> Call EApiFanEnable to enable fan PWM\n");
                printf("                  <param_1> -> PWM cycle\n");
                printf("                  <param_2> -> PWM dutycycle\n\n");
                printf("           -fd -> Call EApiFanDisable to disable fan PWM\n");
                printf("                  No parameter reqired\n\n");
                printf("           -fg -> Call EApiFanPWMGetCycle to get fan PWM cycle\n");
                printf("                  No parameter reqired\n\n");
                printf("           -fs -> Call EApiFanPWMSetCycle to set fan PWM cycle\n");
                printf("                  <param_1> -> PWM cycle\n\n");
                printf("           -fu -> Call EApiFanPWMGetDutyCycle to get fan PWM dutycycle\n");
                printf("                  No parameter reqired\n\n");
                printf("           -fc -> Call EApiFanPWMSetDutyCycle to set fan PWM dutycycle\n");
                printf("                  <param_1> -> PWM cycle\n");
                printf("                  <param_2> -> PWM dutycycle\n");
                printf("\n           -------------------------------------------------------------------------\n\n");
                printf("           -pc -> Call EApiI2CGetBusCap for SPI bus to get the capabilities\n");
                printf("                  No parameter reqired\n\n");
                printf("           -pr -> Call EApiI2CReadTransfer for SPI bus to read byte from register\n");
                printf("                  <param_1> -> Register\n\n");
                printf("           -pw -> Call EApiI2CWriteTransfer for SPI bus to write byte to register xxx\n");
                printf("                  <param_1> -> Register\n");
                printf("                  <param_2> -> Register value to write\n\n");
                printf("           -pe -> Call EApiI2CWriteReadRaw for SPI bus for read and write operations\n");
                printf("                  <param_1> -> Number of bytes to write\n");
                printf("                  <param_2> -> Number of bytes to read\n");
                printf("                  <param_3> ... <param_n> -> Byte to write\n\n");
                printf("\n           -------------------------------------------------------------------------\n\n");
                printf("           -mr -> Call EApiSMBus_RB to read a byte from SMBus\n");
                printf("                  <param_1> -> Address\n");
                printf("                  <param_2> -> Command\n\n");
                printf("           -mw -> Call EApiSMBus_WB to write a byte to SMBus\n");
                printf("                  <param_1> -> Address\n");
                printf("                  <param_2> -> Command\n");
                printf("                  <param_3> -> Byte to write\n\n");
                printf("           -me -> Call EApiSMBus_RW to read a word from SMBus\n");
                printf("                  <param_1> -> Address\n");
                printf("                  <param_2> -> Command\n");
                printf("           -mo -> Call EApiSMBus_WW to write a word to SMBus\n");
                printf("                  <param_1> -> Address\n");
                printf("                  <param_2> -> Command\n");
                printf("                  <param_3> -> Word to write\n\n");
                printf("\n          -------------------------------------------------------------------------\n\n");
                printf("           -rg -> Call EApiGetResetCauses to get reset cause\n");
                printf("                  No parameter reqired\n\n");
                printf("           -rc -> Call EApiClearResetCauses to clear reset causes\n");
                printf("                  <param_1> -> reset causes to clear\n");
                printf("\n          -------------------------------------------------------------------------\n\n");
                return 0;
                break;

                default:
                *command = PARAMETER_ERROR;
                break;
            }
    }
    else
    {
        *command = PARAMETER_ERROR;
    }

    // (int argc, char* argv[], unsigned char* command, unsigned char* param, unsigned char* num_param)

    *num_param = argc - 2;

    //printf("*num_param = %d \n", *num_param);

    for(int a = 2; a < argc; a++)
    {
        unsigned char len = strlen(argv[a]);

        //printf("argv[%d] len = %d \n", a, len);

        param[a - 2] = 0;

        for(int b = 0; b < len; b++)
        {
            if((argv[a][b] < 0x30) || (argv[a][b] > 0x39))
            {
                *num_param = 0;
                //printf("argv[%d][%d] = 0x%02X   *num_param = %d \n",a,b,argv[a][b], *num_param);
                return 0;
           }

            //printf("argv[%d][%d] = %02X , len %d , potenza(10, len - b - 1) = %d \n", a, b, argv[a][b], len, elev_pot(10, len - b - 1));
            param[a - 2] += (argv[a][b] - 0x30) * elev_pot(10, len - b - 1);
        }
        //printf("param[%d] = %d \n", a - 2, param[a - 2]);
    }

    return 0;
}

int main(int argc,char* argv[])
{
	char		StringValue[EAPI_MAX_LEN_STRING];
	uint32_t    StringLen;
	uint32_t	Value;

	uint8_t	    Buffer[EAPI_MAX_LEN_STRING];
	uint32_t    Offset;
	uint32_t	BufferCnt;

	uint32_t	DelayTimeout;
	uint32_t	EventTimeout;
	uint32_t	ResetTimeout;
    uint8_t     mode_Event;
    uint8_t     mode_Trigger_Type;
    uint32_t    Event;
    uint8_t     Trigger_Type;
	uint32_t    ReadCnt;
	uint32_t    WriteCnt;

    uint32_t	Dev_Addr;

//    uint8_t     SMBus_Address;
//    uint8_t     SMBus_Cmd;
//    uint8_t     SMBus_Data_Byte;
//    uint16_t    SMBus_Data_Word;

	uint32_t	Inputs;
    uint32_t	Outputs;

    uint8_t param_byte[128];
    uint32_t param[128];

    unsigned char op_type = NO_OP;

    unsigned char command;
    unsigned char num_param;

	uint32_t	BlockLength;

    parser(argc, argv, &command, param, &num_param);

    //printf("command = %d, num_param = %d, param[0] = %d, param[1] = %d, param[2] = %d \n",
    //       command, num_param, param[0], param[1], param[2]);

    //EApiEnableDebugLog(TRUE);
    if(command != PARAMETER_ERROR)
    {
        if(EApiLibInitialize() != EAPI_STATUS_SUCCESS)
         {
           cout << "EApiLibInitialize failed " << endl;
           return false;
        }
    }
    else
    {
        cout << "PARAMETER_ERROR " << endl;
        return false;
    }

    switch(command)
    {
          case HELP:
            return true;
            break;
          case GET_SPEC_VERSION:
            if(EApiBoardGetValue(EAPI_ID_GET_EAPI_SPEC_VERSION, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_GET_EAPI_SPEC_VERSION -> v.%d.%d.%d \n",	((Value & 0xFF000000) >> 24),
																			((Value & 0x00FF0000) >> 16),
																			(Value & 0x0000FFFF));
            else
                cout << "EAPI_ID_GET_EAPI_SPEC_VERSION -> FAILED " << endl;

            return true;
            break;
        case GET_BOARD_PLATFORM_REV:
            if(EApiBoardGetValue(EAPI_ID_BOARD_PLATFORM_REV_VAL, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_BOARD_PLATFORM_REV_VAL -> v.%d.%d.%d \n",	((Value & 0xFF000000) >> 24),
																			((Value & 0x00FF0000) >> 16),
																			(Value & 0x0000FFFF));
            else
                cout << "EAPI_ID_BOARD_PLATFORM_REV_VAL -> FAILED " << endl;

            return true;
            break;
        case GET_BOARD_LIB_VERSION:
            if(EApiBoardGetValue(EAPI_ID_BOARD_LIB_VERSION_VAL, &Value) == EAPI_STATUS_SUCCESS)
                 printf("EAPI_ID_BOARD_LIB_VERSION_VAL -> v.%d.%d.%d \n",	((Value & 0xFF000000) >> 24),
																			((Value & 0x00FF0000) >> 16),
																			(Value & 0x0000FFFF));
            else
                 cout << "EAPI_ID_BOARD_LIB_VERSION_VAL -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_CPU_TEMP:
            if(EApiBoardGetValue(EAPI_ID_HWMON_CPU_TEMP, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_CPU_TEMP -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_CPU_TEMP -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_CHIPSET_TEMP:
            if(EApiBoardGetValue(EAPI_ID_HWMON_CHIPSET_TEMP, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_CHIPSET_TEMP -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_CHIPSET_TEMP -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_FAN_SYSTEM:
            if(EApiBoardGetValue(EAPI_ID_HWMON_FAN_SYSTEM, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_FAN_SYSTEM -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_FAN_SYSTEM -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_FAN_CPU:
            if(EApiBoardGetValue(EAPI_ID_HWMON_FAN_CPU, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_FAN_CPU -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_FAN_CPU -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_VOLTAGE_VCORE:
            if(EApiBoardGetValue(EAPI_ID_HWMON_VOLTAGE_VCORE, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_VOLTAGE_VCORE -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_VOLTAGE_VCORE -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_VOLTAGE_2V5:
            if(EApiBoardGetValue(EAPI_ID_HWMON_VOLTAGE_2V5, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_VOLTAGE_2V5 -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_VOLTAGE_2V5 -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_VOLTAGE_3V3:
            if(EApiBoardGetValue(EAPI_ID_HWMON_VOLTAGE_3V3, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_VOLTAGE_3V3 -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_VOLTAGE_3V3 -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_VOLTAGE_VBAT:
            if(EApiBoardGetValue(EAPI_ID_HWMON_VOLTAGE_VBAT, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_VOLTAGE_VBAT -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_VOLTAGE_VBAT -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_VOLTAGE_5V:
            if(EApiBoardGetValue(EAPI_ID_HWMON_VOLTAGE_5V, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_VOLTAGE_5V -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_VOLTAGE_5V -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_VOLTAGE_5VSB:
            if(EApiBoardGetValue(EAPI_ID_HWMON_VOLTAGE_5VSB, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_VOLTAGE_5VSB -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_VOLTAGE_5VSB -> FAILED " << endl;

            return true;
            break;
        case GET_HWMON_VOLTAGE_12V:
            if(EApiBoardGetValue(EAPI_ID_HWMON_VOLTAGE_12V, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_HWMON_VOLTAGE_12V -> %d \n", Value);
            else
                cout << "EAPI_ID_HWMON_VOLTAGE_12V -> FAILED " << endl;

            return true;
            break;
        case GET_BOARD_BOOT_COUNTER:
            if(EApiBoardGetValue(EAPI_ID_BOARD_BOOT_COUNTER_VAL, &Value) == EAPI_STATUS_SUCCESS)
                printf("EAPI_ID_BOARD_BOOT_COUNTER_VAL -> %d \n", Value);
            else
                cout << "EAPI_ID_BOARD_BOOT_COUNTER_VAL -> FAILED " << endl;

            return true;
            break;
// --------------------------------------------------------------------------------------------------
        case GET_BOARD_MANUFACTURER:
			StringLen = sizeof(StringValue);
            if(EApiBoardGetStringA( EAPI_ID_BOARD_MANUFACTURER_STR, StringValue, &StringLen) == EAPI_STATUS_SUCCESS)
                cout << "EAPI_ID_BOARD_MANUFACTURER_STR -> " << StringValue << endl;
            else
                cout << "EAPI_ID_BOARD_MANUFACTURER_STR -> FAILED" << endl;

            return true;
            break;
        case GET_BOARD_NAME:
			StringLen = sizeof(StringValue);
            if(EApiBoardGetStringA( EAPI_ID_BOARD_NAME_STR, StringValue, &StringLen) == EAPI_STATUS_SUCCESS)
                cout << "EAPI_ID_BOARD_NAME_STR ->  " << StringValue << endl;
            else
                cout << "EAPI_ID_BOARD_NAME_STR -> FAILED " << endl;

            return true;
            break;
        case GET_BOARD_REVISION:
			StringLen = sizeof(StringValue);
            if(EApiBoardGetStringA( EAPI_ID_BOARD_REVISION_STR, StringValue, &StringLen) == EAPI_STATUS_SUCCESS)
                cout << "EAPI_ID_BOARD_REVISION_STR ->  " << StringValue << endl;
            else
                cout << "EAPI_ID_BOARD_REVISION_STR -> FAILED " << endl;

            return true;
            break;
        case GET_BOARD_SERIAL:
			StringLen = sizeof(StringValue);
            if(EApiBoardGetStringA( EAPI_ID_BOARD_SERIAL_STR, StringValue, &StringLen) == EAPI_STATUS_SUCCESS)
                cout << "EAPI_ID_BOARD_SERIAL_STR ->  " << StringValue << endl;
            else
                cout << "EAPI_ID_BOARD_SERIAL_STR -> FAILED " << endl;

            return true;
            break;
        case GET_BOARD_BIOS_REVISION:
			StringLen = sizeof(StringValue);
            if(EApiBoardGetStringA( EAPI_ID_BOARD_BIOS_REVISION_STR, StringValue, &StringLen) == EAPI_STATUS_SUCCESS)
                cout << "EAPI_ID_BOARD_BIOS_REVISION_STR ->  " << StringValue << endl;
            else
                cout << "EAPI_ID_BOARD_BIOS_REVISION_STR -> FAILED " << endl;

            return true;
            break;
        case GET_BOARD_HW_REVISION:
			StringLen = sizeof(StringValue);
            if(EApiBoardGetStringA( EAPI_ID_BOARD_HW_REVISION_STR, StringValue, &StringLen) == EAPI_STATUS_SUCCESS)
                cout << "EAPI_ID_BOARD_HW_REVISION_STR -> " << StringValue << endl;
            else
                cout << "EAPI_ID_BOARD_HW_REVISION_STR -> FAILED " << endl;

            return true;
            break;
        case GET_BOARD_PLATFORM_TYPE:
			StringLen = sizeof(StringValue);
            if(EApiBoardGetStringA( EAPI_ID_BOARD_PLATFORM_TYPE_STR, StringValue, &StringLen) == EAPI_STATUS_SUCCESS)
                cout << "EAPI_ID_BOARD_PLATFORM_TYPE_STR ->  " << StringValue << endl;
            else
                cout << "EAPI_ID_BOARD_PLATFORM_TYPE_STR -> FAILED " << endl;

            return true;
            break;
// --------------------------------------------------------------------------------------------------
        case GET_LCD_BACKLIGHT:
            if(num_param == 0)
            {
                if(EApiVgaGetBacklightBrightness(EAPI_ID_BACKLIGHT_1, &Value) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiVgaGetBacklightBrightness = %d\n", Value);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

        cout << "EApiVgaGetBacklightBrightness -> FAILED" << endl;
            return true;
            break;
        case SET_LCD_BACKLIGHT:
            Value =	param[0];

            if(num_param == 1)
            {
                if(EApiVgaSetBacklightBrightness(EAPI_ID_BACKLIGHT_1, Value) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiVgaSetBacklightBrightness -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiVgaSetBacklightBrightness -> FAILED" << endl;
            return true;
            break;
        case SET_LCD_BCKL_ENA:
            if(param[0] == 0)
                Value = EAPI_BACKLIGHT_SET_OFF;
            else
                Value = EAPI_BACKLIGHT_SET_ON;

            if(num_param == 1)
            {
                if(EApiVgaSetBacklightEnable(EAPI_ID_BACKLIGHT_1, Value) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiVgaSetBacklightEnable -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiVgaSetBacklightEnable -> FAILED" << endl;
            return true;
            break;
// --------------------------------------------------------------------------------------------------
        case NVS_SIZE:
            if(num_param == 0)
            {
                if(EApiStorageCap(EAPI_ID_STORAGE_STD, &Value, &BlockLength) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiStorageCap = %d\n", Value);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiStorageCap -> FAILED" << endl;
            return true;
            break;
        case NVS_READ:
            BufferCnt = param[0];
            Offset = param[1];

            if(num_param == 2)
            {
                if(EApiStorageAreaRead(EAPI_ID_STORAGE_STD, Offset, Buffer, EAPI_MAX_LEN_STRING, BufferCnt) == EAPI_STATUS_SUCCESS)
                {
                    for(unsigned long e = 0; e < BufferCnt; e++)
                        printf("EApiStorageAreaRead -> ReadByte[%lu] = 0x%02X\n", e, Buffer[e]);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiStorageAreaRead -> FAILED" << endl;
            return true;
            break;
         case NVS_WRITE:
            BufferCnt = param[0];
            Offset = param[1];

            for(unsigned short a = 0; a < BufferCnt; a++)
                param_byte[a] = (unsigned char)param[a + 2];

            if(num_param == (2 + BufferCnt))
            {
                if(EApiStorageAreaWrite(EAPI_ID_STORAGE_STD, Offset, param_byte, BufferCnt) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiStorageAreaWrite -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiStorageAreaWrite -> FAILED" << endl;
            return false;
            break;
// --------------------------------------------------------------------------------------------------
        case WDOG_CAP:
			if(num_param == 0)
			{
                if(EApiWDogGetCap(&DelayTimeout, &EventTimeout, &ResetTimeout) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiWDogGetCap.MaxDelay = %d\n", DelayTimeout);
                    printf("EApiWDogGetCap.MaxEventTimeout = %d\n", EventTimeout);
                    printf("EApiWDogGetCap.MaxResetTimeout = %d\n", ResetTimeout);
                    return true;
                }
			}
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiWDogGetCap -> FAILED" << endl;
            return true;
            break;
        case WDOG_TRIGGER:
            if(num_param == 0)
            {
                if(EApiWDogTrigger() == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiWDogTrigger -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiWDogTrigger -> FAILED" << endl;
            return true;
            break;
        case WDOG_START:
            if(num_param == 3)
            {
                DelayTimeout = param[0];

                EventTimeout = param[1];

                ResetTimeout = param[2];

                if(EApiWDogStart(DelayTimeout, EventTimeout, ResetTimeout) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiWDogStart -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiWDogStart -> FAILED" << endl;
            return true;
            break;
        case WDOG_START_CUSTOM:
            if(num_param == 5)
            {
                DelayTimeout =	param[0];

                EventTimeout =	param[1];

                ResetTimeout =	param[2];

                mode_Event = (uint8_t)param[3];

                mode_Trigger_Type =	(uint8_t)param[4];

                switch(mode_Event)
                {
                    case 0:
                        Event = EAPI_WTD_MODE_SIGNAL;
                        break;
                    case 1:
                        Event = EAPI_WTD_MODE_SYSTEM_RESET;
                        break;
                    case 2:
                        Event = EAPI_WTD_MODE_PWRBTN_1SEC;
                        break;
                    case 3:
                        Event = EAPI_WTD_MODE_PWRBTN_4SEC;
                        break;
                    default:
                        cout << "Parameter error" << endl;
                        return false;
                        break;
                }

                switch(mode_Trigger_Type)
                {
                    case 0:
                        Trigger_Type = EAPI_WTD_MODE_ENABLE_SW_TRIGGER;
                        break;
                    case 1:
                        Trigger_Type = EAPI_WTD_MODE_ENABLE_HW_TRIGGER;
                        break;
                    default:
                        cout << "Parameter error" << endl;
                        return false;
                        break;
                }

                if(EApiWDog_SetConfigAndStart(DelayTimeout,
                                              EventTimeout,
                                              ResetTimeout,
                                              Event,
                                              EAPI_WTD_MODE_IN_MSEC,
                                              EAPI_WTD_MODE_IN_MSEC,
                                              Trigger_Type) == EAPI_STATUS_SUCCESS);
                {
                    cout << "EApiWDog_SetConfigAndStart -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiWDog_SetConfigAndStart -> FAILED" << endl;
            return true;
            break;
        case WDOG_STOP:
            if(num_param == 0)
            {
                if(EApiWDogStop() == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiWDogStop -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiWDogStop -> FAILED" << endl;
            return true;
            break;
// --------------------------------------------------------------------------------------------------
        case I2C_CAP:
            if(num_param == 1)
            {
                if(EApiI2CGetBusCap(param[0], &Value) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiI2CGetBusCap -> 0x%02X\n", Value);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CGetBusCap -> FAILED" << endl;
            return true;
            break;
        case I2C_WRITE_READ_RAW:

            Dev_Addr = (uint8_t)param[1];
            WriteCnt = (uint8_t)param[2];
            ReadCnt  = (uint8_t)param[3];

            for(unsigned short a = 0; a < WriteCnt; a++)
                param_byte[a] = (unsigned char)param[a + 4];

            if(num_param == (4 + WriteCnt))
            {
                if(EApiI2CWriteReadRaw(param[0], Dev_Addr, param_byte, WriteCnt, Buffer, sizeof(Buffer), ReadCnt) == EAPI_STATUS_SUCCESS)
                {
                    if(ReadCnt != 0)
                        printf("EApiI2CWriteReadRaw -> ReadByte[0] = 0x%02X\n",Buffer[0]);
                        else
                        printf("EApiI2CWriteReadRaw -> OK\n");
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CWriteReadRaw -> FAILED" << endl;
            return true;
            break;
        case I2C_READ:
            Dev_Addr = (uint8_t)param[1];
            Offset	 = (uint8_t)param[2];

            if(num_param == 3)
            {
                if(EApiI2CReadTransfer(param[0], Dev_Addr, Offset, Buffer, sizeof(Buffer), 1) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiI2CReadTransfer -> ReadByte = 0x%02X\n", Buffer[0]);

                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CReadTransfer -> FAILED" << endl;
            return true;
            break;
        case I2C_WRITE:
            Dev_Addr = (uint8_t)param[1];
            Offset   = (uint8_t)param[2];
            param_byte[0] = (uint8_t)param[3];

            if(num_param == 4)
            {
                 if(EApiI2CWriteTransfer(param[0], Dev_Addr, Offset, param_byte, 1) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiI2CWrite -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CWrite -> FAILED" << endl;
            return true;
            break;
        case I2C_PROBE:
            Dev_Addr = (uint8_t)param[1];

			if(num_param == 2)
            {
                if(EApiI2CProbeDevice(param[0], Dev_Addr) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiI2CProbeDevice -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CProbeDevice -> FAILED" << endl;
            return true;
            break;

// --------------------------------------------------------------------------------------------------
        case SPI_CAP:
            if(num_param == 0)
            {
                if(EApiI2CGetBusCap(EAPI_ID_I2C_SPI, &Value) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiI2CGetBusCap -> 0x%02X\n", Value);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CGetBusCap -> FAILED" << endl;
            return true;
            break;
        case SPI_WRITE_READ_RAW:
            Dev_Addr = 0;
            WriteCnt = (uint8_t)param[0];
            ReadCnt  = (uint8_t)param[1];

            for(unsigned short a = 0; a < WriteCnt; a++)
                param_byte[a] = (unsigned char)param[a + 2];

            if(num_param == (2 + WriteCnt))
            {
                if(EApiI2CWriteReadRaw(EAPI_ID_I2C_SPI, Dev_Addr, param_byte, WriteCnt, Buffer, sizeof(Buffer), ReadCnt) == EAPI_STATUS_SUCCESS)
                {
                    if(ReadCnt != 0)
                        printf("EApiI2CWriteReadRaw -> ReadByte[0] = 0x%02X\n",Buffer[0]);
                        else
                        printf("EApiI2CWriteReadRaw -> OK\n");
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CWriteReadRaw -> FAILED" << endl;
            return true;
            break;
        case SPI_READ:
            Dev_Addr = 0;
            Offset	 = (uint8_t)param[0];

            if(num_param == 1)
            {
                if(EApiI2CReadTransfer(EAPI_ID_I2C_SPI, Dev_Addr, Offset, Buffer, sizeof(Buffer), 1) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiI2CReadTransfer -> ReadByte = 0x%02X\n", Buffer[0]);

                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CReadTransfer -> FAILED" << endl;
            return true;
            break;
        case SPI_WRITE:
            Dev_Addr      = 0;
            Offset        = (uint8_t)param[0];
            param_byte[0] = (uint8_t)param[1];

            if(num_param == 2)
            {
                 if(EApiI2CWriteTransfer(EAPI_ID_I2C_SPI, Dev_Addr, Offset, param_byte, 1) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiI2CWrite -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiI2CWrite -> FAILED" << endl;
            return true;
            break;

// --------------------------------------------------------------------------------------------------
 /*
        case SMBUS_READ_BYTE:
            SMBus_Address   = (uint8_t)param[0];
            SMBus_Cmd       = (uint8_t)param[1];

            if(num_param == 2)
            {
                if(EApiSMBus_RB(SMBus_Address, SMBus_Cmd, &SMBus_Data_Byte) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiSMBus_RB -> ReadByte[0] = 0x%02X\n", SMBus_Data_Byte);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiSMBus_RB -> FAILED" << endl;
            return true;
            break;

        case SMBUS_WRITE_BYTE:
            SMBus_Address   = (uint8_t)param[0];
            SMBus_Cmd       = (uint8_t)param[1];
            SMBus_Data_Byte = (uint8_t)param[2];

            if(num_param == 3)
            {
                if(EApiSMBus_WB(SMBus_Address, SMBus_Cmd, SMBus_Data_Byte) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiSMBus_WB -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiSMBus_WB -> FAILED" << endl;
            return true;
            break;

        case SMBUS_READ_WORD:
            SMBus_Address   = (uint8_t)param[0];
            SMBus_Cmd       = (uint8_t)param[1];

            if(num_param == 2)
            {
                if(EApiSMBus_RW(SMBus_Address, SMBus_Cmd, &SMBus_Data_Word) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiSMBus_RW -> ReadByte[0] = 0x%04X\n", SMBus_Data_Word);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiSMBus_RW -> FAILED" << endl;
            return true;
            break;

        case SMBUS_WRITE_WORD:
            SMBus_Address   = (uint8_t)param[0];
            SMBus_Cmd       = (uint8_t)param[1];
            SMBus_Data_Word = (uint16_t)param[2];

            if(num_param == 3)
            {
                if(EApiSMBus_WW(SMBus_Address, SMBus_Cmd, SMBus_Data_Word) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiSMBus_WW -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiSMBus_WW -> FAILED" << endl;
            return true;
            break;
*/
// --------------------------------------------------------------------------------------------------
        case GPIO_GET_DIRECTION_CAP:
            if(num_param == 2)
            {
                EApiSetCarrier((uint8_t)param[0]);

                if(EApiGPIOGetDirectionCaps(param[1], &Inputs, &Outputs) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiGPIOGetDirectionCaps Inputs = 0x%08X\n", Inputs);
                    printf("EApiGPIOGetDirectionCaps Outputs = 0x%08X\n", Outputs);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiGPIOGetDirectionCaps -> FAILED" << endl;
            return true;
            break;
        case GPIO_GET_DIRECTION:
            if(num_param == 2)
            {
                uint32_t Bitmask = 0, Inputs, Outputs;

                EApiSetCarrier((uint8_t)param[0]);

                if(EApiGPIOGetDirectionCaps(EAPI_ID_GPIO_BANK00, &Inputs, &Outputs) == EAPI_STATUS_SUCCESS)
                    Bitmask = Inputs | Outputs;

                if(EApiGPIOGetDirection(param[1], Bitmask, &Value) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiGPIOGetDirection Directions = 0x%08X\n", Value);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiGPIOGetDirection -> FAILED" << endl;
            return true;
            break;
        case GPIO_SET_DIRECTION:
            Value =	param[2];
            if(num_param == 3)
            {
                uint32_t Bitmask = 0, Inputs, Outputs;

                EApiSetCarrier((uint8_t)param[0]);

                if(EApiGPIOGetDirectionCaps(EAPI_ID_GPIO_BANK00, &Inputs, &Outputs) == EAPI_STATUS_SUCCESS)
                    Bitmask = Inputs | Outputs;

                if(EApiGPIOSetDirection(param[1], Bitmask, Value) == EAPI_STATUS_SUCCESS)
                {
                   cout << "EApiGPIOSetDirection -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiGPIOSetDirection -> FAILED" << endl;
            return true;
            break;
        case GPIO_GET_LEVEL:
            if(num_param == 2)
            {
                uint32_t Bitmask = 0, Inputs, Outputs;

                EApiSetCarrier((uint8_t)param[0]);

                if(EApiGPIOGetDirectionCaps(EAPI_ID_GPIO_BANK00, &Inputs, &Outputs) == EAPI_STATUS_SUCCESS)
                    Bitmask = Inputs | Outputs;

                if(EApiGPIOGetLevel(param[1], Bitmask, &Value) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiGPIOGetLevel Level = 0x%08X\n", Value);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiGPIOGetLevel -> FAILED" << endl;
            return true;
            break;
        case GPIO_SET_LEVEL:
            Value =	param[2];
            if(num_param == 3)
            {
                uint32_t Bitmask = 0, Inputs, Outputs;

                EApiSetCarrier((uint8_t)param[0]);

                if(EApiGPIOGetDirectionCaps(EAPI_ID_GPIO_BANK00, &Inputs, &Outputs) == EAPI_STATUS_SUCCESS)
                    Bitmask = Inputs | Outputs;

                if(EApiGPIOSetLevel(param[1], Bitmask, Value) == EAPI_STATUS_SUCCESS)
                {
                   cout << "EApiGPIOSetLevel -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiGPIOSetLevel -> FAILED" << endl;
            return true;
            break;
        case GPIO_GET_SERIAL_CFG:
            if(num_param == 1)
            {
                EApiSetCarrier((uint8_t)param[0]);

                if(EApiGPIOGetSerialCfg(Buffer) == EAPI_STATUS_SUCCESS)
                {
                    printf("EAPIGPIOGetSerialCfg = 0x%02X\n", Buffer[0]);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EAPIGPIOGetSerialCfg -> FAILED" << endl;
            return true;
            break;
        case GPIO_SET_SERIAL_CFG:
            if(num_param == 2)
            {
                EApiSetCarrier((uint8_t)param[0]);

                if(EApiGPIOSetSerialCfg((uint8_t)param[1]) == EAPI_STATUS_SUCCESS)
                {
                   cout << "EAPIGPIOSetSerialCfg -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EAPIGPIOSetSerialCfg -> FAILED" << endl;
            return true;
            break;
// --------------------------------------------------------------------------------------------------
       case FAN_ENABLE:
            if(num_param == 2)
            {
                uint16_t freq = (uint16_t)param[0];
                if(EApiFanEnable(/*FAN_FREQUENCY_5MHZ,*/ freq, (uint8_t)param[1]) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiFanEnable -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiFanEnable -> error" << endl;
            return true;
            break;
        case FAN_DISABLE:
            if(num_param == 0)
            {
                if(EApiFanDisable() == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiFanDisable -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiFanDisable -> error" << endl;
            return true;
            break;
        case FAN_GET_CYCLE:
            if(num_param == 0)
            {
                if(EApiFanPWMGetCycle(&Value) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiFanPWMGetCycle = 0x%02X\n", Value);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiFanPWMGetCycle -> error" << endl;
            return true;
            break;
        case FAN_SET_CYCLE:
            if(num_param == 1)
            {
                uint16_t freq = (uint16_t)param[0];
                if(EApiFanPWMSetCycle(freq) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiFanPWMSetCycle -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiFanPWMSetCycle -> error" << endl;
            return true;
            break;
         case FAN_GET_DUTY:
            if(num_param == 0)
            {
                if(EApiFanPWMGetDutyCycle(Buffer) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiFanPWMGetDutyCycle = 0x%02X\n", Buffer[0]);
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiFanPWMGetDutyCycle -> error" << endl;
            return true;
            break;
        case FAN_SET_DUTY:
            if(num_param == 2)
            {
                uint16_t freq = (uint16_t)param[0];
                if(EApiFanPWMSetDutyCycle(freq, (uint8_t)param[1]) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiFanPWMSetDutyCycle -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiFanPWMSetDutyCycle -> error" << endl;
            return true;
            break;
// --------------------------------------------------------------------------------------------------
        case GET_RESET_CAUSE:
            if(num_param == 0)
            {
                uint32_t ResetCauses;
                if(EApiGetResetCauses(&ResetCauses) == EAPI_STATUS_SUCCESS)
                {
                    printf("EApiGetResetCauses = 0x%08X\n", ResetCauses);
                    if(ResetCauses & RESET_CAUSE_RESET)
                        printf("RESET_CAUSE_RESET\n");
                    if(ResetCauses & RESET_CAUSE_WDT)
                        printf("RESET_CAUSE_WDT\n");
                    if(ResetCauses & RESET_CAUSE_PWFAIL)
                        printf("RESET_CAUSE_PWFAIL\n");
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiGetResetCauses -> error" << endl;
            return true;
            break;
        case CLEAR_RESET_CAUSE:
            if(num_param == 1)
            {
                if(EApiClearResetCauses(param[0]) == EAPI_STATUS_SUCCESS)
                {
                    cout << "EApiClearResetCauses -> OK" << endl;
                    return true;
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            cout << "EApiClearResetCauses -> error" << endl;
            return true;
            break;
// --------------------------------------------------------------------------------------------------
       case TEST_01:
            if(num_param == 1)
            {
                uint32_t  loop = 0;
                for(loop = 0; loop < param[0]; loop++)
                {
                    uint32_t  StorageSize = 0;
                    if(EApiStorageCap(EAPI_ID_STORAGE_STD, &StorageSize, &BlockLength ) != EAPI_STATUS_SUCCESS)
                    {
                        cout << "EApiStorageCap -> failed" << endl;
                        return false;
                    }

                    BufferCnt = 1;
                    for(Offset = 0; Offset < StorageSize; Offset++)
                    {
                        param_byte[Offset] = (rand() % 255);
                        //printf("TEST_01 - write=0x%02X at %02d-> OK\n", param_byte[Offset], Offset);
                        if(EApiStorageAreaWrite(EAPI_ID_STORAGE_STD,  Offset, param_byte + Offset, BufferCnt) != EAPI_STATUS_SUCCESS)
                        {
                            printf("EApiStorageAreaWrite failed on offset = %02d\n", Offset);
                            return false;
                        }
                    }

                    for(Offset = 0; Offset < StorageSize; Offset++)
                    {
                        if(EApiStorageAreaRead(EAPI_ID_STORAGE_STD, Offset, Buffer, EAPI_MAX_LEN_STRING, BufferCnt) != EAPI_STATUS_SUCCESS)
                        {
                            printf("EApiStorageAreaRead failed on offset = %02d\n", Offset);
                            return false;
                        }

                        if(Buffer[0] != param_byte[Offset])
                        {
                            printf("Matching failed on Offset = %02d write = 0x%02X read = 0x%02X\n", Offset, param_byte[Offset], Buffer[Offset]);
                            return false;
                        }
                    }

                    printf("TEST_01 - loop=%02d -> OK\n", loop);
                }
            }
            else
            {
                cout << "Parameter error" << endl;
                return false;
            }

            return true;
            break;

        default:
            cout << "PARAMETER_ERROR" << endl;
            return false;
            break;
    }

    return 0;

    if(argc > 1)
    {

    }
    else
    {
        cout << "No param" << endl;
        op_type = NO_OP;
        return 0;
    }

    if(op_type == NO_OP)
    {
        cout << "NO correct operation selected" << endl;
        return 0;
    }

    if(argv[2][0] != '/' && argv[2][0] != '-')
    {
        cout << "Not valid parameter selected" << endl;
        return 0;
    }

    cOption = (char)tolower(argv[2][1]);

    switch(cOption)
    {

    }

    if(op_type == EAPI_LIBRARY)
    {
        cout << "op_type == EAPI_LIBRARY" << argv[1] << endl;
    }

    return 0;
}

